@@ -3,6 +3,7 @@ import matplotlib.pyplot as plt
 import scipy.stats
 from . import matcher
 from scipy.stats import t
+import scipy
 import math
 import pandas as pd
 
@@ -135,8 +136,16 @@ def HotellingT2(cluster1, cluster2):
         raise ValueError("The Hotelling T2 squared statistic is not applicable when the sum of the sample sizes minus "
                          "2 is less than the dimensionality.")
 
-    C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
-    C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+    if scipy.sparse.issparse(cluster1.X):
+        C1Mean = cluster1.X.mean(axis=0)
+    else:
+        C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
+
+    if scipy.sparse.issparse(cluster2.X):
+        C2Mean = cluster2.X.mean(axis=0)
+    else:
+        C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+
     MeanDiff = (C1Mean - C2Mean)  # *patterns_filtered.X[34, :]
 
     # covariance matrices and transform to pooled
@@ -187,8 +196,19 @@ def BonferroniCorrectedDifferenceMeans(cluster1, cluster2, alpha, varName, verbo
     :return: A dataframe of genes as index with their respective confidence intervals in columns low and high.
     """
     dimensionality = cluster1.shape[1]
-    C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
-    C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+
+    if scipy.sparse.issparse(cluster1.X):
+        C1Mean = cluster1.X.mean(axis=0)
+    else:
+        C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
+
+    if scipy.sparse.issparse(cluster2.X):
+        C2Mean = cluster2.X.mean(axis=0)
+    else:
+        C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+
+    # C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
+    # C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
     MeanDiff = (C1Mean - C2Mean)
     plusminus = np.zeros((2, MeanDiff.shape[1]))
     n1 = cluster1.shape[0]
@@ -266,8 +286,18 @@ def projectionDriver(patterns_filtered, cluster1, cluster2, alpha, varName, feat
         raise ValueError("Invalid feature number. Must be between 0 and " + str(patterns_filtered.shape[0] + "."))
 
     dimensionality = cluster1.shape[1]
-    C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
-    C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+    if scipy.sparse.issparse(cluster1.X):
+        C1Mean = cluster1.X.mean(axis=0)
+    else:
+        C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
+
+    if scipy.sparse.issparse(cluster2.X):
+        C2Mean = cluster2.X.mean(axis=0)
+    else:
+        C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
+
+    # C1Mean = np.mean(cluster1.X, axis=0, keepdims=True)
+    # C2Mean = np.mean(cluster2.X, axis=0, keepdims=True)
 
     MeanDiff = (C1Mean - C2Mean)
     feature = patterns_filtered.X[(featureNumber - 1), :]
@@ -311,7 +341,7 @@ def projectionDriver(patterns_filtered, cluster1, cluster2, alpha, varName, feat
             gene = cluster1.var[varName][i]
             list1.append(gene)
 
-    standardBon = BonferroniCorrectedDifferenceMeans(cluster1, cluster2, alpha, varName, verbose=613)
+    standardBon = BonferroniCorrectedDifferenceMeans(cluster1, cluster2, alpha, varName, verbose=613, display=False)
 
     driverBon = pd.DataFrame(np.transpose(plusminus), index=cluster1.var[varName], columns=['Low', 'High'])
 
